 /**
 * @file tal_network_recovery.h
 * @brief Common process - network recovery
 * @version 1.0.0
 * @date 2021-05-01
 *
 * @copyright Copyright 2021-2022 Tuya Inc. All Rights Reserved.
 *
 */

#ifndef __TAL_NETWORK_RECOVERY_H__
#define __TAL_NETWORK_RECOVERY_H__


#ifdef __cplusplus
    extern "C" {
#endif

#include "tuya_cloud_types.h"

/**
 * @brief zigbee network recovery disable
 *
 * @param[in] en: TRUE disables the network recovery 
 * @note the default is enabled network recovery
 * @return  none
 */
VOID_T tal_zg_nwk_recovery_disable(BOOL_T en);

/**
 * @brief zigbee network recovery start
 * it must be used in the no network state
 *
 * @param[in] none
 * @return  none
 */
VOID_T tal_zg_nwk_recovery_start(VOID_T);

/**
 * @brief zigbee network recovery information save
 *
 * @param[in] none
 * @return  TRUE when save info success or FALSE
 */
BOOL_T tal_zg_nwk_recovery_info_save(VOID_T);

/**
 * @brief zigbee network recovery information clear
 *
 * @param[in] none
 * @return  TRUE when clear info success or FALSE
 */
BOOL_T tal_zg_nwk_recovery_info_clear(VOID_T);

/**
 * @brief zigbee network recovery times clear
 *
 * @param[in] none
 * @return  none
 */
VOID_T tal_zg_nwk_recovery_times_clear(VOID_T);

/**
 * @brief zigbee network recovery is available
 *
 * @param[in] none
 * @return  TRUE: available
 */
BOOL_T tal_zg_nwk_recovery_is_available(VOID_T);

#ifdef __cplusplus
}
#endif

#endif

