/*
 * @Author: Deven
 * @Email: liming@tuya.com
 * @LastEditors: Please set LastEditors
 * @FileName: &file name&
 * @Description: 
 * @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
 * @Company: http://www.tuya.com
 * @Date: 2020-11-07 17:16:29
 * @LastEditTime: 2021-03-19 10:34:26
 */

#ifndef _PBUF_H__
#define _PBUF_H__


#include "tkl_system.h"


#ifdef __cplusplus
extern "C" {
#endif


typedef enum {
    NO_SEND = 0,
    SEND_SUCESS,
}SEND_T;

typedef struct {
	VOID_T* (*malloc_fn)(size_t num);
	VOID_T (*free_fn)(VOID_T *ptr);
}mem_t;

typedef enum {
  PBUF_MAC = 0 + 24,
  PBUF_RAW = 0
} pbuf_layer;

struct pbuf {
    /* next pbuf in singly linked pbuf chain */
    struct pbuf *next;
    /* pointer to the actual data in the buffer */
    VOID_T *payload;
    /* the total length of data contained in the current pbuf and all subsequent pbufs */
    UINT16_T tot_len;
    /* length of this buffer */
    UINT16_T len;
    /* hight 4 bits are used for sending status,low 4 bits are used for priority */
    UINT8_T flag;
    /*
     * the reference count always equals the number of pointers
     * that refer to this pbuf. This can be pointers from an application,
     * the stack itself, or pbuf->next pointers from a chain.
     */
    UINT8_T ref;
    /* data management id,use for or sending management */
    UINT8_T data_id;
    /* For incoming packets, this contains the input netif's index */
    UINT8_T card_idx;
    /* send retry times */
    UINT16_T send_retry;
    UINT16_T resv; 
};

#define PBUF_STRUCT_SIZE (sizeof(struct pbuf))

#define SEND_STATUS_GET(pbuf) \
    ((pbuf->flag & 0xF0) >> 4)
    
#define SEND_STATUS_SET(pbuf, status) \
    (pbuf->flag |= (status << 4))

#define PBUF_PRIORITY_GET(pbuf) \
    (pbuf->flag & 0x0F)
    
#define PBUF_PRIORITY_SET(pbuf, status) \
    (pbuf->flag |= (status & 0x0F))

VOID_T mem_init(mem_t* mem);
VOID_T ty_free(VOID_T *p);
VOID_T* ty_malloc(UINT32_T size);

UINT8_T pbuf_free(struct pbuf* p);
struct pbuf* pbuf_alloc(UINT8_T offset, UINT16_T len);
VOID_T pbuf_ref_increase(struct pbuf* p);
UINT16_T pbuf_counter(const struct pbuf* p);
VOID_T pbuf_cat(struct pbuf* front, struct pbuf* after);
VOID_T pbuf_chain(struct pbuf* front, struct pbuf* after);
struct pbuf* pbuf_dechain(struct pbuf *p);
INT8_T pbuf_add_header(struct pbuf* p, UINT16_T hdr_size_inc);
INT8_T pbuf_remove_header(struct pbuf* p, UINT16_T hdr_size_dec);
INT8_T pbuf_take(struct pbuf *buf, const VOID_T *p_data, UINT16_T len);
UINT16_T pbuf_copy_to_swap(const struct pbuf *buf, VOID_T *swap, UINT16_T len, UINT16_T offset);

#ifdef __cplusplus
}
#endif

#endif  // end _MAC_NET_H__
