/********************************************************************************************************
 * @file    tlPrintf.h
 *
 * @brief   This is the header file for tlPrintf
 *
 * @author  Driver & Zigbee Group
 * @date    2021
 *
 * @par     Copyright (c) 2021, Telink Semiconductor (Shanghai) Co., Ltd. ("TELINK")
 *          All rights reserved.
 *
 *          Licensed under the Apache License, Version 2.0 (the "License");
 *          you may not use this file except in compliance with the License.
 *          You may obtain a copy of the License at
 *
 *              http://www.apache.org/licenses/LICENSE-2.0
 *
 *          Unless required by applicable law or agreed to in writing, software
 *          distributed under the License is distributed on an "AS IS" BASIS,
 *          WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *          See the License for the specific language governing permissions and
 *          limitations under the License.
 *
 *******************************************************************************************************/

#ifndef TLPRINTF_H
#define TLPRINTF_H

#include <stdarg.h>
#include "app_cfg.h"

// #define UART_TEST_MODE 1


int Tl_printf(const char *format, ...);

#if (UART_PRINTF_MODE || USB_PRINTF_MODE)
#if (UART_TEST_MODE)
	#define APP_PRINTF(format, ...)
	#define tkl_dbg_printf(format, ...) 
	#define app_printf(format, ...)
	#define printf(format, ...)
	#define TRACE
	#define	DEBUG(compileFlag, ...)
	#define DEBUG_ARRAY(compileFlag, arrayAddr, len)
	#define TEST_PRINTF(format, ...) 					Tl_printf("["__FILE__"][Line: %d][%s]: "format"\n", __LINE__, __func__, ##__VA_ARGS__)
	#define test_printf 								Tl_printf
#else
	#define APP_PRINTF(format, ...) 					Tl_printf("["__FILE__"][Line: %d][%s]: "format"\n", __LINE__, __func__, ##__VA_ARGS__)
	#define tkl_dbg_printf 								Tl_printf
	#define app_printf 									Tl_printf
	#define printf										Tl_printf
	#define TEST_PRINTF(format, ...) 					Tl_printf("["__FILE__"][Line: %d][%s]: "format"\n", __LINE__, __func__, ##__VA_ARGS__)
	#define test_printf 								Tl_printf
	#define TRACE										printf
	#define	DEBUG(compileFlag, ...)						do{	\
															if(compileFlag) TRACE(__VA_ARGS__);		\
														}while(0)

	#define DEBUG_ARRAY(compileFlag, arrayAddr, len)	do{	\
															if(compileFlag){											\
																TRACE("*********************************\n");			\
																unsigned char i = 0;									\
																do{														\
																	TRACE(" %x", ((unsigned char *)arrayAddr)[i++]);	\
																}while(i < len);										\
																TRACE("\n*********************************\n");			\
															}															\
														}while(0)
#endif
#else
	#define APP_PRINTF(format, ...)
	#define tkl_dbg_printf(format, ...) 
	#define app_printf(format, ...)
	#define printf(format, ...)
	#define TEST_PRINTF(format, ...)	
	#define test_printf(format, ...) 																	
	#define TRACE
	#define	DEBUG(compileFlag, ...)
	#define DEBUG_ARRAY(compileFlag, arrayAddr, len)

#endif

#endif


