#include "i2c.h"
#include "tkl_board.h"
#include "tkl_i2c.h"


/*********************************************************************
 * LOCAL CONSTANT
 */

/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */

STATIC UINT32_T tkl_baudrate = 0;

STATIC UINT32_T tlsr_i2c_gpio_group_list[] = {
    I2C_GPIO_GROUP_C2C3, I2C_GPIO_GROUP_C0C1, I2C_GPIO_GROUP_B6D7, I2C_GPIO_GROUP_A3A4
};

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */


STATIC I2C_GPIO_GroupTypeDef tkl_i2c_channel_to_i2c_gpio_group(UINT32_T port)
{
    if(port >= (sizeof(tlsr_i2c_gpio_group_list)/sizeof(tlsr_i2c_gpio_group_list[0]))) {
        return (I2C_GPIO_GroupTypeDef)0xFFFFFFFF;
    }
    UINT32_T tlsr_i2c_gpio_group = tlsr_i2c_gpio_group_list[port];
    return tlsr_i2c_gpio_group;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_init(TUYA_I2C_NUM_E port, CONST TUYA_IIC_BASE_CFG_T *cfg)
{
    OPERATE_RET ret = OPRT_OK;
    I2C_GPIO_GroupTypeDef tlsr_i2c_gpio_group = tkl_i2c_channel_to_i2c_gpio_group(port);

    switch(cfg->speed){
        case TUYA_IIC_BUS_SPEED_100K:
            tkl_baudrate = (100*1000);
        break;
        case TUYA_IIC_BUS_SPEED_400K:
            tkl_baudrate = (400*1000);
        break;
        case TUYA_IIC_BUS_SPEED_1M:
            tkl_baudrate = (1000*1000);
        break;
        case TUYA_IIC_BUS_SPEED_3_4M:
            tkl_baudrate = (3400*1000);
        break;
    }

    i2c_gpio_set(tlsr_i2c_gpio_group);
    i2c_master_init((UINT8_T)(CLOCK_SYS_CLOCK_HZ/(4*tkl_baudrate)));
    
    return ret;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_deinit(TUYA_I2C_NUM_E port)
{
    return OPRT_OK;
}

/**
 * @brief i2c irq init
 * NOTE: call this API will not enable interrupt
 * 
 * @param[in] port: i2c port, id index starts at 0
 * @param[in] cb:  i2c irq cb
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_irq_init(TUYA_I2C_NUM_E port, CONST TUYA_I2C_IRQ_CB *cb)
{
    return OPRT_NOT_SUPPORTED;
}

/**
 * @brief i2c irq enable
 * 
 * @param[in] port: i2c port id, id index starts at 0
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_irq_enable(TUYA_I2C_NUM_E port)
{
    return OPRT_NOT_SUPPORTED;
}

/**
 * @brief i2c irq disable
 * 
 * @param[in] port: i2c port id, id index starts at 0
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_irq_disable(TUYA_I2C_NUM_E port)
{
    return OPRT_NOT_SUPPORTED;
}

/**
 * @brief i2c master send
 * 
 * @param[in] port: i2c port
 * @param[in] dev_addr: iic addrress of slave device.
 * @param[in] data: i2c data to send
 * @param[in] size: Number of data items to send
 * @param[in] xfer_pending: xfer_pending: TRUE : not send stop condition, FALSE : send stop condition.
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_master_send(TUYA_I2C_NUM_E port, UINT16_T dev_addr, CONST VOID_T *data, UINT32_T size, BOOL_T xfer_pending)
{
    return OPRT_NOT_SUPPORTED;
}

/**
 * @brief i2c master recv
 * 
 * @param[in] port: i2c port
 * @param[in] dev_addr: iic addrress of slave device.
 * @param[in] data: i2c buf to recv
 * @param[in] size: Number of data items to receive
 * @param[in] xfer_pending: TRUE : not send stop condition, FALSE : send stop condition.
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_master_receive(TUYA_I2C_NUM_E port, UINT16_T dev_addr, VOID *data, UINT32_T size, BOOL_T xfer_pending)
{
    return OPRT_NOT_SUPPORTED;
}

/**
 * @brief i2c slave
 * 
 * @param[in] port: i2c port
 * @param[in] dev_addr: slave device addr
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_set_slave_addr(TUYA_I2C_NUM_E port, UINT16_T dev_addr)
{
    return OPRT_NOT_SUPPORTED;
}

/**
 * @brief i2c slave send
 * 
 * @param[in] port: i2c port
 * @param[in] data: i2c buf to send
 * @param[in] size: Number of data items to send
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_slave_send(TUYA_I2C_NUM_E port, CONST VOID *data, UINT32_T size)
{
    return OPRT_NOT_SUPPORTED;
}

/**
 * @brief IIC slave receive, Start receiving data as IIC Slave.
 * 
 * @param[in] port: i2c port
 * @param[in] data: Pointer to buffer for data to receive from IIC Master
 * @param[in] size: Number of data items to receive
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_slave_receive(TUYA_I2C_NUM_E port, VOID *data, UINT32_T size)
{
    return OPRT_NOT_SUPPORTED;
}

/**
 * @brief IIC get status.
 * 
 * @param[in] port: i2c port
 * @param[out]  TUYA_IIC_STATUS_T
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE TUYA_IIC_STATUS_T tkl_i2c_get_status(TUYA_I2C_NUM_E port)
{
    TUYA_IIC_STATUS_T status;
    return status;
}

/**
 * @brief i2c's reset
 * 
 * @param[in] port: i2c port number
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET  tkl_i2c_reset(TUYA_I2C_NUM_E port)
{
    return OPRT_NOT_SUPPORTED;
}

/**
 * @brief i2c transferred data count.
 * 
 * @param[in] port: i2c port id, id index starts at 0
 *
 * @return >=0,number of currently transferred data items. <0,err. 
 * tkl_i2c_master_send:number of data bytes transmitted and acknowledged
 * tkl_i2c_master_receive:number of data bytes received
 * tkl_i2c_slave_send:number of data bytes transmitted
 * tkl_i2c_slave_receive:number of data bytes received and acknowledged
 */
TUYA_WEAK_ATTRIBUTE INT32_T tkl_i2c_get_data_count(TUYA_I2C_NUM_E port)
{
    return OPRT_NOT_SUPPORTED;
}

/**
 * @brief i2c ioctl
 *
 * @param[in]       cmd     user def
 * @param[in]       args    args associated with the command
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_i2c_ioctl(UINT32_T cmd,  VOID *args)
{
    return OPRT_NOT_SUPPORTED;
}
