/**
* @file tkl_spi.c
* @brief Common process - adapter the spi api
* @version 0.1
* @date 2021-08-06
*
* @copyright Copyright 2020-2021 Tuya Inc. All Rights Reserved.
*
*/

#include "spi.h"
#include "gpio.h"
#include "tkl_board.h"
#include "tkl_spi.h"

UINT32_T cs_tlsr_pin = 0;

TUYA_WEAK_ATTRIBUTE SPI_ModeTypeDef tkl_spi_mode_transform(TUYA_SPI_MODE_E mode){
    switch(mode){
        case TUYA_SPI_MODE0:
            return SPI_MODE0;
        break;
        case TUYA_SPI_MODE1:
            return SPI_MODE1;
        break;
        case TUYA_SPI_MODE2:
            return SPI_MODE2;
        break;
        case TUYA_SPI_MODE3:
            return SPI_MODE3;
        break;
        default:
            return SPI_MODE0;
    }
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_spi_mapping_to_gpio(UINT32_T port)
{
    if(port > 1) {
        return OPRT_NOT_SUPPORTED;
    }

    if(0 == port){
        spi_master_gpio_set(SPI_GPIO_GROUP_A2A3A4D6);
        cs_tlsr_pin = GPIO_PD6;
    }
    else{
        spi_master_gpio_set(SPI_GPIO_GROUP_B6B7D2D7);
        cs_tlsr_pin = GPIO_PD2;
    }

    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_spi_init(TUYA_SPI_NUM_E port, CONST TUYA_SPI_BASE_CFG_T *cfg){
    if(port > 1) {
        return OPRT_NOT_SUPPORTED;
    }

    if(cfg->databits != TUYA_SPI_DATA_BIT8) {
        return OPRT_NOT_SUPPORTED;
    }

    if(cfg->type != TUYA_SPI_AUTO_TYPE) {
        return OPRT_NOT_SUPPORTED;
    }

    spi_master_init((unsigned char)(CLOCK_SYS_CLOCK_HZ/(2*cfg->freq_hz)-1), tkl_spi_mode_transform(cfg->mode));
    tkl_spi_mapping_to_gpio(port);
    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_spi_deinit(TUYA_SPI_NUM_E port){
    
    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_spi_xfer(TUYA_SPI_NUM_E port, VOID_T* send_buf, VOID_T* receive_buf, UINT32_T length){
    if((port != 0)) {
        return OPRT_NOT_SUPPORTED;
    }

    if(send_buf == NULL) {
        return OPRT_NOT_SUPPORTED;
    }

    if(receive_buf == NULL) {
        return OPRT_NOT_SUPPORTED;
    }

    spi_read(send_buf, length, receive_buf, length, cs_tlsr_pin);
    return OPRT_OK;
}


