/*
 * tkl_platform_types.h
 * Copyright(C),2018-2020,www.tuya.comm
*/

#ifndef __TKL_PLATFORM_TYPES_H__
#define __TKL_PLATFORM_TYPES_H__

#include "tuya_cloud_types.h"
#include "zcl_include.h"


#ifdef __cplusplus
extern "C" {
#endif

#define TUYA_MANAGE_NUMBER			    0x55AA5AA5
#define NWK_JOIN_EVT_CHECK_PERIOD       1

typedef struct {
    af_simple_descriptor_t desc;
    UINT8_T cluster_sum;
    zcl_specClusterInfo_t *cluster_list;
}__ep_info_t;


#ifdef MCU_CORE_8258
#define APP_GPIO_PA0    TUYA_GPIO_NUM_0
#define APP_GPIO_PA1    TUYA_GPIO_NUM_1
#define APP_GPIO_PA2    TUYA_GPIO_NUM_2
#define APP_GPIO_PA3    TUYA_GPIO_NUM_3
#define APP_GPIO_PA4    TUYA_GPIO_NUM_4
#define APP_GPIO_PA5    TUYA_GPIO_NUM_5
#define APP_GPIO_PA6    TUYA_GPIO_NUM_6
#define APP_GPIO_PA7    TUYA_GPIO_NUM_7

#define APP_GPIO_PB0    TUYA_GPIO_NUM_8
#define APP_GPIO_PB1    TUYA_GPIO_NUM_9
#define APP_GPIO_PB2    TUYA_GPIO_NUM_10
#define APP_GPIO_PB3    TUYA_GPIO_NUM_11
#define APP_GPIO_PB4    TUYA_GPIO_NUM_12
#define APP_GPIO_PB5    TUYA_GPIO_NUM_13
#define APP_GPIO_PB6    TUYA_GPIO_NUM_14
#define APP_GPIO_PB7    TUYA_GPIO_NUM_15

#define APP_GPIO_PC0    TUYA_GPIO_NUM_16
#define APP_GPIO_PC1    TUYA_GPIO_NUM_17
#define APP_GPIO_PC2    TUYA_GPIO_NUM_18
#define APP_GPIO_PC3    TUYA_GPIO_NUM_19
#define APP_GPIO_PC4    TUYA_GPIO_NUM_20
#define APP_GPIO_PC5    TUYA_GPIO_NUM_21
#define APP_GPIO_PC6    TUYA_GPIO_NUM_22
#define APP_GPIO_PC7    TUYA_GPIO_NUM_23

#define APP_GPIO_PD0    TUYA_GPIO_NUM_24
#define APP_GPIO_PD1    TUYA_GPIO_NUM_25
#define APP_GPIO_PD2    TUYA_GPIO_NUM_26
#define APP_GPIO_PD3    TUYA_GPIO_NUM_27
#define APP_GPIO_PD4    TUYA_GPIO_NUM_28
#define APP_GPIO_PD5    TUYA_GPIO_NUM_29
#define APP_GPIO_PD6    TUYA_GPIO_NUM_30
#define APP_GPIO_PD7    TUYA_GPIO_NUM_31

#define APP_GPIO_PE0    TUYA_GPIO_NUM_32
#define APP_GPIO_PE1    TUYA_GPIO_NUM_33
#define APP_GPIO_PE2    TUYA_GPIO_NUM_34
#define APP_GPIO_PE3    TUYA_GPIO_NUM_35

#define APP_GPIO_NULL   0xFF
#endif


/**
 * @enum GPIO port number
 */
typedef enum {
    PORT_A = 0x00, ///< portA
    PORT_B,        ///< portB
    PORT_C,
    PORT_D,
    PORT_E,
    PORT_F,
    PORT_H,
    PORT_I,
    PORT_J,
    PORT_K,
}GPIO_PORT_T;


/**
 * @enum GPIO pin number
 */
typedef enum {
    PIN_0 = 0x00, ///< PIN_0
    PIN_1,        ///< PIN_1
    PIN_2,
    PIN_3,
    PIN_4,
    PIN_5,
    PIN_6,
    PIN_7,
    PIN_8,
    PIN_9,
    PIN_10,
    PIN_11,
    PIN_12,
    PIN_13,
    PIN_14,
    PIN_15,
}GPIO_PIN_T;


/**
 * @enum GPIO mode definition
 */
typedef enum {
    GPIO_MODE_INPUT_HIGH_IMPEDANCE = 0, ///< input mode: high impedance
    GPIO_MODE_INPUT_PULL,               ///< input mode: pull(up or down)
    GPIO_MODE_OUTPUT_PP,                ///< output mode: Push-pull 
    GPIO_MODE_OUTPUT_OD,                ///< output mode: Open drain 
    GPIO_MODE_OUTPUT_OD_PULL_UP,
    GPIO_MODE_OUTPUT_OD_PULL_DOWN,
}GPIO_MODE_T;


/**
 * @enum GPIO output/input configuration 
 */
typedef enum {
    GPIO_DOUT_LOW = 0,  ///< ouput low or input pull down
    GPIO_DOUT_HIGH = 1, ///< ouput high or input pull up
}GPIO_DOUT_T;


/**
 * @enum GPIO interrupt mode 
 */
typedef enum {
    GPIO_LEVEL_LOW = 0, ///< Drop edge triggers interrupt
    GPIO_LEVEL_HIGH,    ///< Rising edge triggers interrupt
    GPIO_LEVEL_ALL,     ///< Rising and Drop edge triggers interrupt
}GPIO_LEVEL_T;



typedef struct {
    GPIO_PORT_T port;
    GPIO_PIN_T pin;
}GPIO_PORT_PIN_T;


/**
 * @note GPIO configuration description
 */
typedef struct {
    GPIO_PORT_T port;
    GPIO_PIN_T pin;
    GPIO_MODE_T mode;
    GPIO_DOUT_T out;
    GPIO_LEVEL_T drive_flag; ///< the effectively level (0 or 1)
} gpio_config_t;


/**
 * @enum UART id
 */
typedef enum {
    UART_ID_UART0 = 0,  /**< UART0 */
    UART_ID_UART1,      /**< UART1 */
}UART_ID_T;


/**
 * @enum UART GPIO select
 */
typedef enum {
    UART_PIN_TYPE_DEFAULT = 0, ///<default uart is PA0,PA1
    UART_PIN_TYPE_CONFIG       ///<user config uart port and pin 
}UART_PIN_TYPE_T;

/**
 * @enum GPIO function redirection
 */
typedef enum {
    LOC_0 = 0, ///< LOC_0
    LOC_1,     ///< LOC_1
    LOC_2,
    LOC_3,
    LOC_4,
    LOC_5,
    LOC_6,
    LOC_7,
    LOC_8,
    LOC_9,
    LOC_10,
    LOC_11,
    LOC_12,
    LOC_13,
    LOC_14,
    LOC_15,
    LOC_16,
    LOC_17,
    LOC_18,
    LOC_19,
    LOC_20,
    LOC_21,
    LOC_22,
    LOC_23,
    LOC_24,
    LOC_25,
    LOC_26,
    LOC_27,
    LOC_28,
    LOC_29,
    LOC_30,
    LOC_31,
}GPIO_LOC_T;


/**
 * @enum UART parity type
 */
typedef enum {
    USART_PARITY_NONE = 0,    /**< No parity. */
    USART_PARITY_EVEN,      /**< Even parity. */
    USART_PARITY_ODD,      /**< Odd parity. */
} USART_PARITY_T;


/**
 * @enum UART stop-bits 
 */
typedef enum {
    USART_STOPBITS_HALF = 0,        /**< 0.5 stop bits. */
    USART_STOPBITS_ONE,         /**< 1 stop bits. */
    USART_STOPBITS_ONEANDAHALF, /**< 1.5 stop bits. */
    USART_STOPBITS_TWO          /**< 2 stop bits. */
} USART_STOPBITS_T;


/**
 * @enum UART data-bits 
 */
typedef enum {
    USART_DATABITS_8BIT = 0,        /**< 8 bits databits. */
    USART_DATABITS_9BIT,         /**< 9 bits databits. */
} USART_DATABITS_T;


/**
 * @note This is a callback function style for uart rx.
 */
typedef void(*uart_callback)(UINT8_T *data, UINT16_T len);

/**
 * @enum UART base configuration
 */
typedef struct {
    UART_ID_T uart_id; 
    UART_PIN_TYPE_T pin_type; 
    GPIO_PORT_PIN_T tx;
    GPIO_PORT_PIN_T rx;
    GPIO_LOC_T tx_loc;
    GPIO_LOC_T rx_loc;
    UINT32_T baud_rate;
    USART_PARITY_T parity;
    USART_STOPBITS_T stop_bits;
    USART_DATABITS_T data_bits;
    uart_callback func;
}user_uart_config_t;


//gpio up down resistor level
typedef enum{
    GPIO_RESISTOR_FLOAT = 0x00,
    GPIO_RESISTOR_PULLUP_1M,
    GPIO_RESISTOR_PULLUP_10K,
    GPIO_RESISTOR_PULLDOWN_100K,
} GPIO_PULL_TYPE_T;


#define DEV_UART_WAKE_UP_BITMAPS    0x00000001
#define DEV_TIMER0_WAKE_UP_BITMAPS  0x00000002
#define DEV_TIMER1_WAKE_UP_BITMAPS  0x00000004
#define DEV_TIMER2_WAKE_UP_BITMAPS  0x00000008
#define DEV_TIMER3_WAKE_UP_BITMAPS  0x00000010
#define DEV_PWM_WAKE_UP_BITMAPS     0x00000020
#define DEV_GPIO_WAKE_UP_BITMAPS    0x00000040

#ifdef __cplusplus
}
#endif

#endif
